//////////////////////////////////////////////////////////////////////
// sangoma_interface.cpp: interface for Sangoma API driver.
//
// Author	:	David Rokhvarg	<davidr@sangoma.com>
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include "sangoma_interface.h"

#define DBG_IFACE	if(0)printf
#define INFO_IFACE	if(1)printf
#define ERR_IFACE	printf("Error: %s(): line:%d :", __FUNCTION__, __LINE__);printf
#define WARN_IFACE	printf("Warning: %s(): line:%d :", __FUNCTION__, __LINE__);printf

#define IFACE_FUNC() if(0)printf("%s():line:%d\n", __FUNCTION__, __LINE__)

#define DO_COMMAND(wan_udp)	DoManagementCommand(sangoma_dev, &wan_udp);

sangoma_interface::sangoma_interface(int wanpipe_number, int interface_number)
{
	DBG_IFACE("%s()\n", __FUNCTION__);

	memset(device_name, 0x00, sizeof(device_name));
	memset(&wan_udp, 0x00, sizeof(wan_udp));
	memset(&tdm_api_cmd, 0x00, sizeof(tdm_api_cmd));
	memset(&wp_api, 0x00, sizeof(wp_api));

	WanpipeNumber = wanpipe_number;
	InterfaceNumber = interface_number;

	//Form the Interface Name from Wanpipe Number and Interface Index (i.e. wanpipe1_if1).
	//(This Interface Name can be used for debugging.)
	_snprintf(device_name, sizeof(device_name), WP_INTERFACE_NAME_FORM, wanpipe_number, interface_number);

	DBG_IFACE("Using Device Name: %s\n", device_name);

	//////////////////////////////////////////////////////////////////
	//IOCTL management structures and variables
	wan_protocol = 0;
	adapter_type = 0;
}

sangoma_interface::~sangoma_interface()
{
	DBG_IFACE("%s()\n", __FUNCTION__);
	cleanup();
}

int sangoma_interface::init()
{
	DBG_IFACE("%s()\n", __FUNCTION__);

	////////////////////////////////////////////////////////////////////////////
	//open handle for reading and writing data, for events reception and other commands
	sangoma_dev = open_api_device();
    if (sangoma_dev == INVALID_HANDLE_VALUE){
		DBG_IFACE( "Unable to open %s for Rx/Tx!\n", device_name);
		return 1;
	}

	////////////////////////////////////////////////////////////////////////////
	//get current protocol
	if(get_wan_config() == WAN_FALSE){
		DBG_IFACE( "Failed to get current protocol!\n");
		return 1;
	}

	////////////////////////////////////////////////////////////////////////////
	//get Front End Type (T1/E1/Analog...)
	if (get_fe_type(&adapter_type) == WAN_FALSE){
		DBG_IFACE( "Failed to get Front End Type!\n");
		return 1;
	}

	////////////////////////////////////////////////////////////////////////////
	//may need interface configuration, so get it now
	if(get_interface_configuration(&wanif_conf_struct)){
		DBG_IFACE( "Failed to get Interface Configuration!\n");
		return 1;
	}

	return 0;
}

int sangoma_interface::print_api_info()
{
	if(sangoma_dev == INVALID_HANDLE_VALUE){
		ERR_IFACE("Error: initialization incomplete!\n");
		return 1;
	}

	INFO_IFACE(WP_API_CFG_FORMAT,
		device_name,	
		SDLA_DECODE_WAN_MEDIA(adapter_type), 
		wp_decode_submedia(wanif_conf_struct.media, wanif_conf_struct.sub_media),	
		wanif_conf_struct.line_mode,
		SDLA_DECODE_USEDBY_FIELD(wanif_conf_struct.usedby)
		);

	return 0;
}

int sangoma_interface::print_channels_info()
{
	char cfg_active_chans[40], ec_chans[40], user_cfg_chans[40];

	if(sangoma_dev == INVALID_HANDLE_VALUE){
		ERR_IFACE("Error: initialization incomplete!\n");
		return 1;
	}

	wp_snprintf(cfg_active_chans, sizeof(cfg_active_chans), "0x%08X",	wanif_conf_struct.active_ch);
	wp_snprintf(ec_chans, sizeof(ec_chans), "0x%08X",					wanif_conf_struct.ec_active_ch);
	wp_snprintf(user_cfg_chans, sizeof(user_cfg_chans), "0x%08X",		wanif_conf_struct.cfg_active_ch);

	INFO_IFACE(WP_CHAN_CFG_FORMAT,
		device_name,	
		cfg_active_chans,	//Configued Active Channels
		ec_chans,			//Echo Canceller Channels
		user_cfg_chans		//User Specified Channels\n\tduring Port Config\t
		);

	return 0;
}

sng_fd_t sangoma_interface::open_api_device()
{
	DBG_IFACE("%s(): WanpipeNumber: %d, InterfaceNumber: %d\n", __FUNCTION__, WanpipeNumber, InterfaceNumber);
	return sangoma_open_api_span_chan(WanpipeNumber, InterfaceNumber);
}

int sangoma_interface::get_wan_config()
{
	int err = WAN_TRUE;

	/* Get Protocol type */
	wan_udp.wan_udphdr_command	= WAN_GET_PROTOCOL;
	wan_udp.wan_udphdr_data_len = 0;

	DO_COMMAND(wan_udp);
	if (wan_udp.wan_udphdr_return_code){
		ERR_IFACE( "Error: Command WANPIPEMON_GET_PROTOCOL failed! return code: 0x%X\n",
			wan_udp.wan_udphdr_return_code);
		return WAN_FALSE;
	}

	wan_protocol = wan_udp.wan_udphdr_data[0];

	return err;
}

int sangoma_interface::get_fe_type(unsigned char* adapter_type)
{
	int err = WAN_TRUE;

	/* Read Adapter Type */
	wan_udp.wan_udphdr_command = WAN_GET_MEDIA_TYPE;
	wan_udp.wan_udphdr_data[0] = WAN_MEDIA_NONE;
	wan_udp.wan_udphdr_data_len = 0;

	DO_COMMAND(wan_udp);
	if(wan_udp.wan_udphdr_return_code){
		ERR_IFACE( "Error: Command WANPIPEMON_GET_MEDIA_TYPE failed! return code: 0x%X",
			wan_udp.wan_udphdr_return_code);
		return WAN_FALSE;
	}

	*adapter_type =	get_wan_udphdr_data_byte(0);

	return err;
}

//return POINTER to data at offset 'off'
unsigned char* sangoma_interface::get_wan_udphdr_data_ptr(unsigned char off)
{
	unsigned char *p_data = (unsigned char*)&wan_udp.wan_udphdr_data[0];
	p_data += off;
	return p_data;
}

unsigned char sangoma_interface::set_wan_udphdr_data_byte(unsigned char off, unsigned char data)
{
	unsigned char *p_data = (unsigned char*)&wan_udp.wan_udphdr_data[0];
	p_data[off] = data;
	return 0;
}

//return DATA at offset 'off'
unsigned char sangoma_interface::get_wan_udphdr_data_byte(unsigned char off)
{
	unsigned char *p_data = (unsigned char*)&wan_udp.wan_udphdr_data[0];
	return p_data[off];
}

unsigned char sangoma_interface::get_adapter_type()
{
	return adapter_type;
}

unsigned int sangoma_interface::get_sub_media()
{
	return wanif_conf_struct.sub_media;
}

int sangoma_interface::get_interface_configuration(if_cfg_t *wanif_conf_ptr)
{
	wan_udp.wan_udphdr_command = WANPIPEMON_READ_CONFIGURATION;
	wan_udp.wan_udphdr_data_len = sizeof(if_cfg_t);

	memset(wanif_conf_ptr, 0x00, sizeof(if_cfg_t));

	DO_COMMAND(wan_udp);
	if(wan_udp.wan_udphdr_return_code){
		ERR_IFACE( "Error: command READ_CONFIGURATION failed!\n");
		return 1;
	}
	memcpy(wanif_conf_ptr, get_wan_udphdr_data_ptr(0), sizeof(if_cfg_t));

	return 0;
}

int sangoma_interface::get_open_handles_counter()
{
	return sangoma_get_open_cnt(sangoma_dev,&wp_api);
}

//It is very important to close ALL open
//handles, because the API will continue
//receive data until the LAST handel is closed.
void sangoma_interface::cleanup()
{
	DBG_IFACE("%s()\n", __FUNCTION__);

	if(sangoma_dev != INVALID_HANDLE_VALUE){
		DBG_IFACE( "Closing 'sangoma_dev' fd.\n");
		sangoma_close(&sangoma_dev);
		sangoma_dev = INVALID_HANDLE_VALUE;
	}
}

int sangoma_interface::DoManagementCommand(sng_fd_t fd, wan_udp_hdr_t* wan_udp)
{
	return sangoma_mgmt_cmd(fd, wan_udp);
}

int sangoma_interface::tdmv_api_ioctl(wanpipe_api_cmd_t *api_cmd)
{
	int err;

	memset(&wp_api, 0x00, sizeof(wp_api));

	memcpy(&wp_api.wp_cmd, api_cmd, sizeof(wanpipe_api_cmd_t));

	err = sangoma_cmd_exec(sangoma_dev, &wp_api);

	memcpy(api_cmd, &wp_api.wp_cmd, sizeof(wanpipe_api_cmd_t));

	return err;
}
