//////////////////////////////////////////////////////////////////////
// sangoma_port.cpp: implementation of the sangoma_port class.
//
// Author	:	David Rokhvarg	<davidr@sangoma.com>
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include "sangoma_port.h"

#if 0
# define DBG_SNG_PORT	printf("%s():line:%d: ", __FUNCTION__, __LINE__);printf
# define _DBG_SNG_PORT	printf
#else
# define DBG_SNG_PORT
# define _DBG_SNG_PORT
#endif

#define ERR_SNG_PORT	printf("Error: %s():line:%d: ", __FUNCTION__, __LINE__);printf
#define _ERR_SNG_PORT	printf

#define SNG_PORT_FUNC()	DBG_SNG_PORT("\n")	

#pragma comment( lib, "libsangoma" )	/* import functions from libsangoma.lib */

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

sangoma_port::sangoma_port()
{
	SNG_PORT_FUNC();
	wp_handle = INVALID_HANDLE_VALUE;
}

sangoma_port::~sangoma_port()
{
	SNG_PORT_FUNC();
	cleanup();
}

//////////////////////////////////////////////////////////////////////
// other functions
//////////////////////////////////////////////////////////////////////

int sangoma_port::init(uint16_t wanpipe_number)
{
//	SNG_PORT_FUNC();
	wp_number = wanpipe_number;
	wp_handle = sangoma_open_driver_ctrl(wanpipe_number);

	if(wp_handle == INVALID_HANDLE_VALUE){
		//ERR_SNG_PORT("failed to open wanpipe%d!!\n", wp_number);
		return 1;
	}

	return 0;
}

void sangoma_port::cleanup()
{
	if(wp_handle != INVALID_HANDLE_VALUE){
		sangoma_close(&wp_handle);
		wp_handle = INVALID_HANDLE_VALUE;
	}
}

int sangoma_port::get_hardware_info(hardware_info_t *hardware_info)
{
	port_management_struct_t port_mgmnt;
	int rc;
	
	memset(&port_mgmnt, 0x00, sizeof(port_mgmnt));

	rc = sangoma_driver_get_hw_info(wp_handle, &port_mgmnt, wp_number);
	if (rc) {
		ERR_SNG_PORT("sangoma_driver_get_hw_info() failed for wanpipe%d!\n", wp_number);
		return rc;
	}

	memcpy(hardware_info, port_mgmnt.data, sizeof(*hardware_info));

	return 0;
}

int sangoma_port::start_port()
{
	port_management_struct_t port_mgmnt;
	int rc;

	memset(&port_mgmnt, 0x00, sizeof(port_mgmnt));

	rc = sangoma_driver_port_start(wp_handle,&port_mgmnt,wp_number);
	if(rc){
		WANCFG_BREAK_POINT();
		DBG_SNG_PORT("rc: %s (%d)\n", SDLA_DECODE_SANG_STATUS(rc), rc);
		return rc;
	}

	return port_mgmnt.operation_status;
}

int sangoma_port::stop_port()
{
	port_management_struct_t port_mgmnt;
	int rc;

	memset(&port_mgmnt, 0x00, sizeof(port_mgmnt));

	rc = sangoma_driver_port_stop(wp_handle, &port_mgmnt, wp_number);
	if(rc){
		WANCFG_BREAK_POINT();
		return rc;
	}

	if(port_mgmnt.operation_status){
		if(SANG_STATUS_CAN_NOT_STOP_DEVICE_WHEN_ALREADY_STOPPED == port_mgmnt.operation_status){
			port_mgmnt.operation_status = SANG_STATUS_SUCCESS;
		}else{
			WANCFG_BREAK_POINT();
		}
	}

	return port_mgmnt.operation_status;
}

sng_fd_t sangoma_port::get_wp_handle()
{
	return this->wp_handle;
}


int sangoma_port::push_a_card_into_wanpipe_info_array(
								wanpipe_instance_info_t *wanpipe_info_array,
								hardware_info_t *new_hw_info)
{
	unsigned int card_ind;
	hardware_info_t *tmp_hw_info;

	//check this card is not in the array already
	for(card_ind = 0; card_ind < MAX_CARDS; card_ind++){
		tmp_hw_info = &wanpipe_info_array[card_ind].hardware_info;

		if(	(new_hw_info->pci_bus_number == tmp_hw_info->pci_bus_number) &&
			(new_hw_info->pci_slot_number == tmp_hw_info->pci_slot_number)){
			//already in array. not an error.
			DBG_SNG_PORT("already in array.\n");
			return 1;
		}
	}

	//find an empty slot in the array and place the new card there
	for(card_ind = 0; card_ind < MAX_CARDS; card_ind++){
		tmp_hw_info = &wanpipe_info_array[card_ind].hardware_info;
		if(tmp_hw_info->card_model == -1){
			//found an empty slot in array
			memcpy(tmp_hw_info, new_hw_info, sizeof(hardware_info_t));
			return 0;
		}
	}

	//too many cards in the computer.
	DBG_SNG_PORT("Warning: number of cards in the computer is greater than the maximum of %d.\n", 
		MAX_CARDS);
	return 2;
}

//
//Scan all running wanpipes, check which ones are on the same card,
//figure out how many physical cards installed.
//
//Returns:	-1 - if error.
//			number of cards, starting from zero.
int sangoma_port::scan_for_sangoma_cards(wanpipe_instance_info_t *wanpipe_info_array, int wanpipe_info_array_length, int *card_model)
{
	uint16_t card_counter = 0, wp_ind;
	hardware_info_t	hardware_info;

	//initialize hardware information array.
	memset(wanpipe_info_array, -1, sizeof(wanpipe_instance_info_t) * wanpipe_info_array_length);

	for(wp_ind = 0; wp_ind < wanpipe_info_array_length; wp_ind++){
		if(init(wp_ind)){
			continue;
		}
		if(get_hardware_info(&hardware_info)){
			card_counter = -1;
			break;
		}

		DBG_SNG_PORT("card_model		: %s (0x%08X)\n",
			SDLA_ADPTR_NAME(hardware_info.card_model), hardware_info.card_model);
		DBG_SNG_PORT("firmware_version\t: 0x%02X\n", hardware_info.firmware_version);
		DBG_SNG_PORT("pci_bus_number\t\t: %d\n", hardware_info.pci_bus_number);
		DBG_SNG_PORT("pci_slot_number\t: %d\n", hardware_info.pci_slot_number);
		DBG_SNG_PORT("port_number\t\t: %d\n", hardware_info.port_number);
		DBG_SNG_PORT("max_hw_ec_chans\t: %d\n", hardware_info.max_hw_ec_chans);
		
		if(	card_model == NULL /* caller is interested in ALL cards, independent of model */ ||
			*card_model == hardware_info.card_model){

			if(push_a_card_into_wanpipe_info_array(wanpipe_info_array, &hardware_info) == 0){
				//At this point we know: 1. WANPIPE number, for example WANPIPE1.
				//						 2. Card type, for example A104.
				//It allows to figure out Device Name of each port on the card.
				//For example: for A104 it will be WANPIPE1, WANPIPE2, WANPIPE3, WANPIPE4
				wanpipe_info_array[card_counter].wanpipe_number = wp_ind;
				DBG_SNG_PORT("wp_ind: %u\n", wp_ind);
				card_counter++;
			}
		}else{
			DBG_SNG_PORT("Found a Card but Card Model is different from what was searched for.\n");
		}

		//For each init() there MUST be cleanup(). Otherwise Open Handles will remain to the Port.
		cleanup();

	}//for()

	return card_counter;
}
