//////////////////////////////////////////////////////////////////////
// wancfglib.cpp : Defines the exported functions for the DLL.
//
// Author	:	David Rokhvarg	<davidr@sangoma.com>
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include "wancfglib.h"
#include "sangoma_port.h"
#include "sangoma_interface.h"
#include "wp_hwprobe.h"
#include "wancfglib_conf_file_reader.h"

#if DBG_FLAG
# define WANCFGLIB_DBG	printf
#else
# define WANCFGLIB_DBG
#endif

int WANCFGLIB_API wancfglib_get_hwprobe_str(char *buf, int buf_size, char verbose)
{
	WANCFGLIB_DBG("%s(): buf: 0x%p, buf_size: %d\n", __FUNCTION__, buf, buf_size);

	wanrouter::wp_hwprobe hwprobe;

	return hwprobe.get_hwprobe_str(buf, buf_size, verbose);
}

int WANCFGLIB_API wancfglib_port_stop(int wanpipe_number)
{
	sangoma_port sng_port;
	int rc;

	WANCFGLIB_DBG("%s(): wanpipe_number: %d\n", __FUNCTION__, wanpipe_number);

	rc = sng_port.init((u16)wanpipe_number);
	if(rc){
		return rc;
	}

	rc = sng_port.stop_port();
	if(rc){
		return rc;
	}

	return 0;
}

int WANCFGLIB_API wancfglib_port_start(int wanpipe_number)
{
	sangoma_port	sng_port;
	port_cfg_t		port_cfg;
	int				rc;

	WANCFGLIB_DBG("%s(): wanpipe_number: %d\n", __FUNCTION__, wanpipe_number);
	
	/////////////////////////////////////////////////////////////////////////////////////////////
	//parse wanpipeX.conf file
	wancfglib_conf_file_reader conf_file_reader(wanpipe_number);
	rc = conf_file_reader.read_conf_file();
	if(rc){
		WANCFG_BREAK_POINT();
		return rc;
	}

	/////////////////////////////////////////////////////////////////////////////////////////////
	//get Port configuration from Conf File Reader
	rc = conf_file_reader.get_port_cfg(&port_cfg);
	if(rc){
		WANCFG_BREAK_POINT();
		return rc;
	}

	rc = sng_port.init((u16)wanpipe_number);
	if(rc){
		WANCFG_BREAK_POINT();
		return rc;
	}

	/////////////////////////////////////////////////////////////////////////////////////////////
	//set volatile configuration
	rc = sangoma_driver_port_set_config(sng_port.get_wp_handle(), &port_cfg, wanpipe_number);
	if(rc){
		WANCFG_BREAK_POINT();
		return rc;
	}
	if(SANG_STATUS_SUCCESS != port_cfg.operation_status){
		WANCFG_BREAK_POINT();
		return port_cfg.operation_status;
	}

	/////////////////////////////////////////////////////////////////////////////////////////////
	//start the port with the new configuration
	rc = sng_port.start_port();
	if(rc){
		WANCFG_BREAK_POINT();
		return rc;
	}

	/////////////////////////////////////////////////////////////////////////////////////////////
	//Some additional configuration may be required for the Interfaces after
	//they were started. For example the HWEC config.
	rc = conf_file_reader.configure_hw_interfaces();
	if(rc){
		WANCFG_BREAK_POINT();
		return rc;
	}

	return rc;
}

/*!
	\fn int WANCFGLIB_API wancfglib_get_wanpipe_list_str(char *buf, int buf_size)
	\brief Get the list of Sangoma Wanpipe devices (Ports on Sangoma cards)
	\param char *buf The output buffer which will contain the list after function returns
	\param int buf_size Size of the output buffer pointed by 'buf' parameter
	\return Number of bytes written to the output buffer. If zero is returned 
*/
int WANCFGLIB_API wancfglib_get_wanpipe_list_str(char *buf, int buf_size)
{
	uint16_t wp_ind;
	sangoma_port sng_port;
	std::string strWpList;
	char cTemp[200];

	memset(buf, 0x00, buf_size);

	for(wp_ind = 0; wp_ind < (WP_MAX_PORTS); wp_ind++){

		if(sng_port.init(wp_ind)){
			//port does not exist
			continue;
		}
		//port exist
		wp_snprintf(cTemp, sizeof(cTemp), "wanpipe%d\n", wp_ind);

		strWpList += cTemp;

		sng_port.cleanup();
	}//for()

	wp_snprintf(buf, buf_size, strWpList.c_str());

	return strlen(strWpList.c_str());
}

int WANCFGLIB_API wancfglib_list_interfaces(int verbose)
{
	uint16_t wp_ind, if_ind, if_counter;
	sangoma_port sng_port;
	sangoma_interface *sng_if;
	std::vector<sangoma_interface*> sng_ifs;
	unsigned int index;

	if_counter = 0;

	////////////////////////////////////////////////////////////////////////////////////
	// build the list of interfaces
	for(wp_ind = 0; wp_ind < (WP_MAX_PORTS); wp_ind++){

		if(sng_port.init(wp_ind)){
			//port does not exist
			continue;
		}
		//port exist, so check what interfaces running on this port
		sng_port.cleanup();

		for(if_ind = 0; if_ind < WP_MAX_INTERFACES; if_ind++){

			sng_if = new sangoma_interface(wp_ind, if_ind);
			if(!sng_if->init()){
				sng_ifs.push_back(sng_if);
			}
		}//for()
	}//for()

	if_counter = sng_ifs.size();

	////////////////////////////////////////////////////////////////////////////////////
	// Print general information about the interfaces:
	printf("\n");
	printf(WP_API_CFG_FORMAT, "Interface",	"Media",	"Media",	"Line",	"API");
	printf(WP_API_CFG_FORMAT, "Name",		"",			"sub-type",	"Mode",	"Mode");
	printf("\n");
	for(index = 0; index < sng_ifs.size(); ++index){

		sng_ifs.at(index)->print_api_info();
	}//for()

	if(verbose){
		////////////////////////////////////////////////////////////////////////////////////
		// Print timeslot/channel information about the interfaces:
		printf("\n");
		printf(WP_CHAN_CFG_FORMAT, "Interface",	 "Configured",	"Echo",			"User");
		printf(WP_CHAN_CFG_FORMAT, "Name",		 "Active",		"Canceller",	"Specified");
		printf(WP_CHAN_CFG_FORMAT, "",			 "Channels",	"Channels",		"Channels");
		printf("\n");
		for(index = 0; index < sng_ifs.size(); ++index){

			sng_ifs.at(index)->print_channels_info();
		}//for()
	}

	//delete all interface objects
	for(index = 0; index < sng_ifs.size(); ++index){

		delete sng_ifs.at(index);
	}

	return if_counter;
}

int WANCFGLIB_API wancfglib_get_wanpipe_status_str(char *buf, int buf_size)
{
	return 0;
}

int WANCFGLIB_API wancfglib_get_wanpipe_instance_info(wanpipe_instance_info_t *wanpipe_info_array, int wanpipe_info_array_length, int *in_card_model)
{
	sangoma_port sng_port;

	return sng_port.scan_for_sangoma_cards(wanpipe_info_array, wanpipe_info_array_length, in_card_model);
}

// The entry point for the DLL.
BOOL APIENTRY DllMain( HMODULE hModule,
                       DWORD  ul_reason_for_call,
                       LPVOID lpReserved
					 )
{
	switch (ul_reason_for_call)
	{
	case DLL_PROCESS_ATTACH:
	case DLL_THREAD_ATTACH:
	case DLL_THREAD_DETACH:
	case DLL_PROCESS_DETACH:
		break;
	}
	return TRUE;
}
